local function makeMaskColorList(mask)
	local uniqueColors = {}
	mask:mapPixel(function(x, y, r, g, b, a)
		if a > 0 then
			local brightness = math.floor((r + g + b) * 255)
			uniqueColors[brightness] = uniqueColors[brightness] or { r, g, b }
		end
		return r, g, b, a
	end)

	local colors = {}
	for _, color in pairs(uniqueColors) do
		table.insert(colors, color)
	end
	-- sort mask colors by brightness
	table.sort(colors, function(a, b) return a[1] + a[2] + a[3] < b[1] + b[2] + b[3] end)
	colors[0] = { -1, -1, -1 } -- item 0 is a "no country" entry
	return colors
end

local function getCountryAt(mask, colors, x, y)
	if x >= 0 and x < mask:getWidth() and y >= 0 and y < mask:getHeight() then
		local r, g, b = mask:getPixel(x, y)
		for index, c in ipairs(colors) do
			if math.abs((c[1] + c[2] + c[3]) - (r + g + b)) < .001 then
				return index;
			end
		end
	end
	return 0
end

local lg = love.graphics
local mapImage, grayMapImage = lg.newImage('map.png'), lg.newImage('map_gray.png')
local mapMask = love.image.newImageData('map_mask.png')
local maskColors = makeMaskColorList(mapMask);

-- shader receives the active country's mask color and discards everything that isn't that country
local highlightShader = lg.newShader([[
	uniform vec3 country;
	uniform Image mask;

	vec4 effect(vec4 col, Image tex, vec2 uv, vec2 fc) {
		vec3 maskValue = Texel(mask, uv).rgb;
		if(distance(maskValue, country) > .001)
			discard;
		return Texel(tex, uv);
	}
]])
highlightShader:send('mask', lg.newImage(mapMask))

local CountryNames = { "Here", "There", "Elsewhere", "Over there", "That place", "Far away", }

function love.draw()
	local selectedCountry = getCountryAt(mapMask, maskColors, love.mouse.getPosition())
	highlightShader:send('country', maskColors[selectedCountry])

	lg.draw(grayMapImage)
	lg.setShader(highlightShader)
	lg.draw(mapImage)
	lg.setShader()
	lg.print(CountryNames[selectedCountry] or "Nothing here", 10, 10)
end
